@@ -5,15 +5,17 @@
  *
  * See src/backend/utils/misc/README for design notes.
  *
- * Portions Copyright (c) 1996-2005, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2009, PostgreSQL Global Development Group
  *
- *	  $PostgreSQL: pgsql/src/include/utils/guc_tables.h,v 1.20 2005/07/14 05:13:44 tgl Exp $
+ *	  $PostgreSQL: pgsql/src/include/utils/guc_tables.h,v 1.46 2009/06/11 14:49:13 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
 #ifndef GUC_TABLES_H
 #define GUC_TABLES_H 1
 
+#include "utils/guc.h"
+
 /*
  * GUC supports these types of variables:
  */
@@ -22,7 +24,8 @@ enum config_type
 	PGC_BOOL,
 	PGC_INT,
 	PGC_REAL,
-	PGC_STRING
+	PGC_STRING,
+	PGC_ENUM
 };
 
 union config_var_value
@@ -31,6 +34,7 @@ union config_var_value
 	int			intval;
 	double		realval;
 	char	   *stringval;
+	int			enumval;
 };
 
 /*
@@ -45,7 +49,6 @@ enum config_group
 	CONN_AUTH_SECURITY,
 	RESOURCES,
 	RESOURCES_MEM,
-	RESOURCES_FSM,
 	RESOURCES_KERNEL,
 	WAL,
 	WAL_SETTINGS,
@@ -77,18 +80,27 @@ enum config_group
 };
 
 /*
- * Stack entry for saving the state of a variable prior to the current
- * transaction
+ * Stack entry for saving the state a variable had prior to an uncommitted
+ * transactional change
  */
+typedef enum
+{
+	/* This is almost GucAction, but we need a fourth state for SET+LOCAL */
+	GUC_SAVE,					/* entry caused by function SET option */
+	GUC_SET,					/* entry caused by plain SET command */
+	GUC_LOCAL,					/* entry caused by SET LOCAL command */
+	GUC_SET_LOCAL				/* entry caused by SET then SET LOCAL */
+} GucStackState;
+
 typedef struct guc_stack
 {
 	struct guc_stack *prev;		/* previous stack item, if any */
-	int			nest_level;		/* nesting depth of cur transaction */
-	int			status;			/* previous status bits, see below */
-	GucSource	tentative_source;		/* source of the tentative_value */
-	GucSource	source;			/* source of the actual value */
-	union config_var_value tentative_val;		/* previous tentative val */
-	union config_var_value value;		/* previous actual value */
+	int			nest_level;		/* nesting depth at which we made entry */
+	GucStackState state;		/* see enum above */
+	GucSource	source;			/* source of the prior value */
+	union config_var_value prior;		/* previous value of variable */
+	union config_var_value masked;		/* SET value in a GUC_SET_LOCAL entry */
+	/* masked value's source must be PGC_S_SESSION, so no need to store it */
 } GucStack;
 
 /*
@@ -111,26 +123,21 @@ struct config_generic
 	enum config_type vartype;	/* type of variable (set only at startup) */
 	int			status;			/* status bits, see below */
 	GucSource	reset_source;	/* source of the reset_value */
-	GucSource	tentative_source;		/* source of the tentative_value */
 	GucSource	source;			/* source of the current actual value */
-	GucStack   *stack;			/* stacked outside-of-transaction states */
+	GucStack   *stack;			/* stacked prior values */
+	char	   *sourcefile;		/* file this settings is from (NULL if not
+								 * file) */
+	int			sourceline;		/* line in source file */
 };
 
-/* bit values in flags field */
-#define GUC_LIST_INPUT			0x0001	/* input can be list format */
-#define GUC_LIST_QUOTE			0x0002	/* double-quote list elements */
-#define GUC_NO_SHOW_ALL			0x0004	/* exclude from SHOW ALL */
-#define GUC_NO_RESET_ALL		0x0008	/* exclude from RESET ALL */
-#define GUC_REPORT				0x0010	/* auto-report changes to client */
-#define GUC_NOT_IN_SAMPLE		0x0020	/* not in postgresql.conf.sample */
-#define GUC_DISALLOW_IN_FILE	0x0040	/* can't set in postgresql.conf */
-#define GUC_CUSTOM_PLACEHOLDER	0x0080	/* placeholder for custom variable */
-#define GUC_SUPERUSER_ONLY		0x0100	/* show only to superusers */
+/* bit values in flags field are defined in guc.h */
 
 /* bit values in status field */
-#define GUC_HAVE_TENTATIVE	0x0001		/* tentative value is defined */
-#define GUC_HAVE_LOCAL		0x0002		/* a SET LOCAL has been executed */
-#define GUC_HAVE_STACK		0x0004		/* we have stacked prior value(s) */
+#define GUC_IS_IN_FILE		0x0001		/* found it in config file */
+/*
+ * Caution: the GUC_IS_IN_FILE bit is transient state for ProcessConfigFile.
+ * Do not assume that its value represents useful information elsewhere.
+ */
 
 
 /* GUC records for specific variable types */
@@ -138,58 +145,66 @@ struct config_generic
 struct config_bool
 {
 	struct config_generic gen;
-	/* these fields must be set correctly in initial value: */
-	/* (all but reset_val are constants) */
+	/* constant fields, must be set correctly in initial value: */
 	bool	   *variable;
-	bool		reset_val;
+	bool		boot_val;
 	GucBoolAssignHook assign_hook;
 	GucShowHook show_hook;
 	/* variable fields, initialized at runtime: */
-	bool		tentative_val;
+	bool		reset_val;
 };
 
 struct config_int
 {
 	struct config_generic gen;
-	/* these fields must be set correctly in initial value: */
-	/* (all but reset_val are constants) */
+	/* constant fields, must be set correctly in initial value: */
 	int		   *variable;
-	int			reset_val;
+	int			boot_val;
 	int			min;
 	int			max;
 	GucIntAssignHook assign_hook;
 	GucShowHook show_hook;
 	/* variable fields, initialized at runtime: */
-	int			tentative_val;
+	int			reset_val;
 };
 
 struct config_real
 {
 	struct config_generic gen;
-	/* these fields must be set correctly in initial value: */
-	/* (all but reset_val are constants) */
+	/* constant fields, must be set correctly in initial value: */
 	double	   *variable;
-	double		reset_val;
+	double		boot_val;
 	double		min;
 	double		max;
 	GucRealAssignHook assign_hook;
 	GucShowHook show_hook;
 	/* variable fields, initialized at runtime: */
-	double		tentative_val;
+	double		reset_val;
 };
 
 struct config_string
 {
 	struct config_generic gen;
-	/* these fields must be set correctly in initial value: */
-	/* (all are constants) */
+	/* constant fields, must be set correctly in initial value: */
 	char	  **variable;
 	const char *boot_val;
 	GucStringAssignHook assign_hook;
 	GucShowHook show_hook;
 	/* variable fields, initialized at runtime: */
 	char	   *reset_val;
-	char	   *tentative_val;
+};
+
+struct config_enum
+{
+	struct config_generic gen;
+	/* constant fields, must be set correctly in initial value: */
+	int		   *variable;
+	int			boot_val;
+	const struct config_enum_entry *options;
+	GucEnumAssignHook assign_hook;
+	GucShowHook show_hook;
+	/* variable fields, initialized at runtime: */
+	int			reset_val;
 };
 
 /* constant tables corresponding to enums above and in guc.h */
@@ -203,4 +218,10 @@ extern struct config_generic **get_guc_variables(void);
 
 extern void build_guc_variables(void);
 
+/* search in enum options */
+extern const char *config_enum_lookup_by_value(struct config_enum * record, int val);
+extern bool config_enum_lookup_by_name(struct config_enum * record,
+						   const char *value, int *retval);
+
+
 #endif   /* GUC_TABLES_H */
